/**
 * jopcodes.c -- Java opcodes.
 *
 * Author: Gaurav Vaish <gvaish@iitk.ac.in>
 *         Gopal V (gopalv82@yahoo.com)
 *
 * (C) Gaurav Vaish, Gopal V; 2002
 */

#include <jopcodes.h>

/**
 * Array of all the opcodes.
 */

opcode_map map[] = {
	{50, "aaload",          NONE},
	{83, "aastore",         NONE},
	{1,  "aconst_null",     NONE},
	{25, "aload",           I},
	{42, "aload_0",         NONE},
	{43, "aload_1",         NONE},
	{44, "aload_2",         NONE},
	{45, "aload_3",         NONE},
	{189,"anewarray",       CLASS},
	{176,"areturn",         NONE},
	{190,"arraylength",     NONE},
	{58, "astore",          I},
	{75, "astore_0",        NONE},
	{76, "astore_1",        NONE},
	{77, "astore_2",        NONE},
	{78, "astore_3",        NONE},
	{191,"athrow",          NONE},
	{51, "baload",          NONE},
	{84, "bastore",         NONE},
	{16, "bipush",          I},
	{202,"breakpoint",      NONE},
	{52, "caload",          NONE},
	{85, "castore",         NONE},
	{192,"checkcast",       CLASS},
	{144,"d2f",             NONE},
	{142,"d2i",             NONE},
	{143,"d2l",             NONE},
	{99, "dadd",            NONE},
	{49, "daload",          NONE},
	{82, "dastore",         NONE},
	{152,"dcmpg",           NONE},
	{151,"dcmpl",           NONE},
	{14, "dconst_0",        NONE},
	{15, "dconst_1",        NONE},
	{111,"ddiv",            NONE},
	{24, "dload",           I},
	{38, "dload_0",         NONE},
	{39, "dload_1",         NONE},
	{40, "dload_2",         NONE},
	{41, "dload_3",         NONE},
	{107,"dmul",            NONE},
	{119,"dneg",            NONE},
	{115,"drem",            NONE},
	{175,"dreturn",         NONE},
	{57, "dstore",          I},
	{71, "dstore_0",        NONE},
	{72, "dstore_1",        NONE},
	{73, "dstore_2",        NONE},
	{74, "dstore_3",        NONE},
	{103,"dsub",            NONE},
	{89, "dup",             NONE},
	{92, "dup2",            NONE},
	{93, "dup2_x1",         NONE},
	{94, "dup2_x2",         NONE},
	{90, "dup_x1",          NONE},
	{91, "dup_x2",          NONE},
	{141,"f2d",             NONE},
	{139,"f2i",             NONE},
	{140,"f2l",             NONE},
	{98, "fadd",            NONE},
	{48, "faload",          NONE},
	{81, "fastore",         NONE},
	{150,"fcmpg",           NONE},
	{149,"fcmpl",           NONE},
	{11, "fconst_0",        NONE},
	{12, "fconst_1",        NONE},
	{13, "fconst_2",        NONE},
	{110,"fdiv",            NONE},
	{23, "fload",           I},
	{34, "fload_0",         NONE},
	{35, "fload_1",         NONE},
	{36, "fload_2",         NONE},
	{37, "fload_3",         NONE},
	{106,"fmul",            NONE},
	{118,"fneg",            NONE},
	{114,"frem",            NONE},
	{174,"freturn",         NONE},
	{56, "fstore",          I},
	{67, "fstore_0",        NONE},
	{68, "fstore_1",        NONE},
	{69, "fstore_2",        NONE},
	{70, "fstore_3",        NONE},
	{102,"fsub",            NONE},
	{180,"getfield",        FIELD},
	{178,"getstatic",       FIELD},
	{167,"goto",            LABEL},
	{200,"goto_w",          LABEL},
	{135,"i2d",             NONE},
	{134,"i2f",             NONE},
	{133,"i2l",             NONE},
	{96, "iadd",            NONE},
	{46, "iaload",          NONE},
	{126,"iand",            NONE},
	{79, "iastore",         NONE},
	{3,  "iconst_0",        NONE},
	{4,  "iconst_1",        NONE},
	{5,  "iconst_2",        NONE},
	{6,  "iconst_3",        NONE},
	{7,  "iconst_4",        NONE},
	{8,  "iconst_5",        NONE},
	{2,  "iconst_m1",       NONE},
	{108,"idiv",            NONE},
	{165,"if_acmpeq",       LABEL},
	{166,"if_acmpne",       LABEL},
	{159,"if_icmpeq",       LABEL},
	{162,"if_icmpge",       LABEL},
	{163,"if_icmpgt",       LABEL},
	{164,"if_icmple",       LABEL},
	{161,"if_icmplt",       LABEL},
	{160,"if_icmpne",       LABEL},
	{153,"ifeq",            LABEL},
	{156,"ifge",            LABEL},
	{157,"ifgt",            LABEL},
	{158,"ifle",            LABEL},
	{155,"iflt",            LABEL},
	{154,"ifne",            LABEL},
	{199,"ifnonnull",       LABEL},
	{198,"ifnull",          LABEL},
	{132,"iinc",            II},
	{21, "iload",           I},
	{26, "iload_0",         NONE},
	{27, "iload_1",         NONE},
	{28, "iload_2",         NONE},
	{29, "iload_3",         NONE},
	{104,"imul",            NONE},
	{116,"ineg",            NONE},
	{193,"instanceof",      CLASS},
	{145,"int2byte",        NONE},
	{146,"int2char",        NONE},
	{147,"int2short",       NONE},
	{145,"i2b",             NONE},
	{146,"i2c",             NONE},
	{147,"i2s",             NONE},
	{185,"invokeinterface", INTERFACE},
	{183,"invokenonvirtual",METHOD},
	{183,"invokespecial",   METHOD},
	{184,"invokestatic",    METHOD},
	{182,"invokevirtual",   METHOD},
	{128,"ior",             NONE},
	{112,"irem",            NONE},
	{172,"ireturn",         NONE},
	{120,"ishl",            NONE},
	{122,"ishr",            NONE},
	{54, "istore",          I},
	{59, "istore_0",        NONE},
	{60, "istore_1",        NONE},
	{61, "istore_2",        NONE},
	{62, "istore_3",        NONE},
	{100,"isub",            NONE},
	{124,"iushr",           NONE},
	{130,"ixor",            NONE},
	{168,"jsr",             LABEL},
	{168,"jsr_w",           LABEL},
	{138,"l2d",             NONE},
	{137,"l2f",             NONE},
	{136,"l2i",             NONE},
	{97, "ladd",            NONE},
	{47, "laload",          NONE},
	{127,"land",            NONE},
	{80, "lastore",         NONE},
	{148,"lcmp",            NONE},
	{9,  "lconst_0",        NONE},
	{10, "lconst_1",        NONE},
	{18, "ldc",             CONSTANT},
	{18, "ldc_w",           CONSTANT},
	{20, "ldc2_w",          BIGCONSTANT},
	{109,"ldiv",            NONE},
	{22, "lload",           I},
	{30, "lload_0",         NONE},
	{31, "lload_1",         NONE},
	{32, "lload_2",         NONE},
	{33, "lload_3",         NONE},
	{105,"lmul",            NONE},
	{117,"lneg",            NONE},
	{171,"lookupswitch",    SWITCH},
	{129,"lor",             NONE},
	{113,"lrem",            NONE},
	{173,"lreturn",         NONE},
	{121,"lshl",            NONE},
	{123,"lshr",            NONE},
	{55, "lstore",          I},
	{63, "lstore_0",        NONE},
	{64, "lstore_1",        NONE},
	{65, "lstore_2",        NONE},
	{66, "lstore_3",        NONE},
	{101,"lsub",            NONE},
	{125,"lushr",           NONE},
	{131,"lxor",            NONE},
	{194,"monitorenter",    NONE},
	{195,"monitorexit",     NONE},
	{197,"multianewarray",  MARRAY},
	{187,"new",             CLASS},
	{188,"newarray",        ARRAYTYPE},
	{0,  "nop",             NONE},
	{87, "pop",             NONE},
	{88, "pop2",            NONE},
	{181,"putfield",        FIELD},
	{179,"putstatic",       FIELD},
	{169,"ret",             I},
	{169,"ret_w",           I},
	{177,"return",          NONE},
	{53, "saload",          NONE},
	{86, "sastore",         NONE},
	{17, "sipush",          I},
	{95, "swap",            NONE},
	{170,"tableswitch",     SWITCH},
	{196,"wide",            IGNORE},
};

/**
 * Returns the index of opcode_map in the map defined above
 * @param opcode Opcode to be searched for.
 * @return Index at which it appears in the array "map"
 */

int search_opcode_map(unsigned char opcode)
{
	int i;
	for(i = 0; i < sizeof(map)/sizeof(opcode_map); i++)
	{
		if(map[i].opcode == opcode) return i;
	}
	return -1;
}

/**
 * Returns an opcode_map for the specified index.
 * @param index Index (base value = 0)
 * @return opcode_map corresponding to the index given
 */


opcode_map get_opcode_map(int index)
{
	return map[index];
}